import re
from us_states import STATES_NORMALIZED, USPS_CHOICES

zip_re = "(?P<zip5>\d{5})(?:\s*-\s*(?P<plus4>\d{4}))?"
state_re = "|".join(u[0] for u in USPS_CHOICES)
city_state_zip_re = "^(?P<city>[\w\.\s]+),\s+(?P<state>({}))\s+(?P<zip>{})$".format(
        state_re, zip_re)

class AddressException(Exception):
    pass

def normalize_zip(code):
    match = re.match(zip_re, code)
    if match.group("plus4"):
        return u"{}-{}".format(match.group("zip5"), match.group("plus4"))
    return match.group("zip5")


def parse_address(address):
    """
    Take a multiline textual address, and return a dictionary of components of the form:
    {
        "name": <string>,
        "organization": <string>,
        "address1": <string>,
        "address2": <string>,
        "address3": <string>,
        "city": <string>,
        "state": <2 letter postal code>,
        "zip": <zip, potentially with +4>
    }
    """
    # Split address string.
    parts = [a.strip() for a in address.strip().split("\n")]
    if len(parts) < 3:
        raise AddressException("Less than 3 lines: {}".format(repr(address)))
    parsed = {
        u"name": parts.pop(0)
    }

    # zip on separate line? put it on previous line for regular parsing.
    if re.match("^{}$".format(zip_re), parts[-1]):
        postcode = parts.pop(-1)
        parts[-1] = "{} {}".format(parts[-1], postcode)

    # Remove U.S.A. country
    if re.match("U\.?S\.?A\.?", parts[-1]):
        parts.pop(-1)

    # Too many lines? augment name.
    if len(parts) > 4:
        parsed["name"] = "{} {}".format(parsed["name"], parts.pop(0))
    # Still too many? Add organization.
    if len(parts) > 4:
        parsed["organization"] = parts.pop(0)
    # Still too many? Give up.
    if len(parts) > 4:
        raise AddressException("Too many lines: {}".format(repr(address)))

    # Now we should only have address1 etc. and city/state/zip line.
    # Pull out address1, address2, etc parts.
    count = 1
    while len(parts) > 1:
        parsed[u"address{}".format(count)] = parts.pop(0)
        count += 1

    city_state_zip = parts[0]
    match = re.match(city_state_zip_re, city_state_zip)
    if match:
        parsed[u"city"] = match.group("city")
        parsed[u"state"] = match.group("state")
        parsed[u"zip"] = normalize_zip(match.group("zip"))
    else:
        # Look for a zip to normalize out.
        search = re.search(zip_re, city_state_zip)
        if search:
            city_state = city_state_zip[0:search.start(1)].strip()
            parsed[u"zip"] = normalize_zip(search.group(0))
        else:
            raise AddressException("Unmatched zip: {}".format(repr(address)))

        # Look for a state to normalize out.
        cs = [a for a in re.split("[^\w.]", re.sub("[\.]", "", city_state)) if a]
        if cs[-1].lower() in STATES_NORMALIZED:
            parsed[u"state"] = unicode(STATES_NORMALIZED[cs[-1].lower()])
            # Replace once from the right.  The tricky [::-1]'s are to flip
            # both the original and repalcement strings, so that we can replace
            # right-to-left, and then flipping the result back around.
            parsed[u"city"] = city_state[::-1].replace(cs[-1][::-1], "", 1)[::-1].rstrip(", ")
        elif " ".join(cs[-2:]).lower() in STATES_NORMALIZED:
            parsed[u"state"] = STATES_NORMALIZED[" ".join(cs[-2:]).lower()]
            parsed[u"city"] = city_state.replace(cs[-1], "").replace(cs[-2], "").rstrip(", ")
        else:
            raise AddressException("Unmatched city/state/zip: {}".format(repr(address)))
    return parsed

def format_address(parsed):
    order = ("name", "organization", "address1", "address2", "address3")
    parts = [parsed[o] for o in order if o in parsed]
    parts.append(u"{}, {}  {}".format(parsed["city"], parsed["state"], parsed["zip"]))
    return u"\n".join(parts)

def _parse_all(addresses):
    parsed = []
    for a in addresses:
        try:
            parsed.append(parse_address(a))
        except AddressException as e:
            print e.message
    return parsed





import os
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="addresscleaner",
    version="0.0.4",
    author="Charlie DeTar",
    author_email="cfd@media.mit.edu",
    url="https://github.com/yourcelf/btb-address-cleaner",
    long_description=read('README.md'),
    description="Utility for normalizing (Prisoner-centric) street addresses",
    license='BSD',
    packages=['addresscleaner'],
)



# From http://django-localflavor.readthedocs.org/en/latest/
"""
A mapping of state misspellings/abbreviations to normalized
abbreviations, and alphabetical lists of US states, territories,
military mail regions and non-US states to which the US provides
postal service.

This exists in this standalone file so that it's only imported into memory
when explicitly needed.
"""

#: The 48 contiguous states, plus the District of Columbia.
CONTIGUOUS_STATES = (
    ('AL', 'Alabama'),
    ('AZ', 'Arizona'),
    ('AR', 'Arkansas'),
    ('CA', 'California'),
    ('CO', 'Colorado'),
    ('CT', 'Connecticut'),
    ('DE', 'Delaware'),
    ('DC', 'District of Columbia'),
    ('FL', 'Florida'),
    ('GA', 'Georgia'),
    ('ID', 'Idaho'),
    ('IL', 'Illinois'),
    ('IN', 'Indiana'),
    ('IA', 'Iowa'),
    ('KS', 'Kansas'),
    ('KY', 'Kentucky'),
    ('LA', 'Louisiana'),
    ('ME', 'Maine'),
    ('MD', 'Maryland'),
    ('MA', 'Massachusetts'),
    ('MI', 'Michigan'),
    ('MN', 'Minnesota'),
    ('MS', 'Mississippi'),
    ('MO', 'Missouri'),
    ('MT', 'Montana'),
    ('NE', 'Nebraska'),
    ('NV', 'Nevada'),
    ('NH', 'New Hampshire'),
    ('NJ', 'New Jersey'),
    ('NM', 'New Mexico'),
    ('NY', 'New York'),
    ('NC', 'North Carolina'),
    ('ND', 'North Dakota'),
    ('OH', 'Ohio'),
    ('OK', 'Oklahoma'),
    ('OR', 'Oregon'),
    ('PA', 'Pennsylvania'),
    ('RI', 'Rhode Island'),
    ('SC', 'South Carolina'),
    ('SD', 'South Dakota'),
    ('TN', 'Tennessee'),
    ('TX', 'Texas'),
    ('UT', 'Utah'),
    ('VT', 'Vermont'),
    ('VA', 'Virginia'),
    ('WA', 'Washington'),
    ('WV', 'West Virginia'),
    ('WI', 'Wisconsin'),
    ('WY', 'Wyoming'),
)

#: All 50 states, plus the District of Columbia.
US_STATES = (
    ('AL', 'Alabama'),
    ('AK', 'Alaska'),
    ('AZ', 'Arizona'),
    ('AR', 'Arkansas'),
    ('CA', 'California'),
    ('CO', 'Colorado'),
    ('CT', 'Connecticut'),
    ('DE', 'Delaware'),
    ('DC', 'District of Columbia'),
    ('FL', 'Florida'),
    ('GA', 'Georgia'),
    ('HI', 'Hawaii'),
    ('ID', 'Idaho'),
    ('IL', 'Illinois'),
    ('IN', 'Indiana'),
    ('IA', 'Iowa'),
    ('KS', 'Kansas'),
    ('KY', 'Kentucky'),
    ('LA', 'Louisiana'),
    ('ME', 'Maine'),
    ('MD', 'Maryland'),
    ('MA', 'Massachusetts'),
    ('MI', 'Michigan'),
    ('MN', 'Minnesota'),
    ('MS', 'Mississippi'),
    ('MO', 'Missouri'),
    ('MT', 'Montana'),
    ('NE', 'Nebraska'),
    ('NV', 'Nevada'),
    ('NH', 'New Hampshire'),
    ('NJ', 'New Jersey'),
    ('NM', 'New Mexico'),
    ('NY', 'New York'),
    ('NC', 'North Carolina'),
    ('ND', 'North Dakota'),
    ('OH', 'Ohio'),
    ('OK', 'Oklahoma'),
    ('OR', 'Oregon'),
    ('PA', 'Pennsylvania'),
    ('RI', 'Rhode Island'),
    ('SC', 'South Carolina'),
    ('SD', 'South Dakota'),
    ('TN', 'Tennessee'),
    ('TX', 'Texas'),
    ('UT', 'Utah'),
    ('VT', 'Vermont'),
    ('VA', 'Virginia'),
    ('WA', 'Washington'),
    ('WV', 'West Virginia'),
    ('WI', 'Wisconsin'),
    ('WY', 'Wyoming'),
)

#: Non-state territories.
US_TERRITORIES = (
    ('AS', 'American Samoa'),
    ('GU', 'Guam'),
    ('MP', 'Northern Mariana Islands'),
    ('PR', 'Puerto Rico'),
    ('VI', 'Virgin Islands'),
)

#: Military postal "states". Note that 'AE' actually encompasses
#: Europe, Canada, Africa and the Middle East.
ARMED_FORCES_STATES = (
    ('AA', 'Armed Forces Americas'),
    ('AE', 'Armed Forces Europe'),
    ('AP', 'Armed Forces Pacific'),
)

#: Non-US locations serviced by USPS (under Compact of Free
#: Association).
COFA_STATES = (
    ('FM', 'Federated States of Micronesia'),
    ('MH', 'Marshall Islands'),
    ('PW', 'Palau'),
)

#: Obsolete abbreviations (no longer US territories/USPS service, or
#: code changed).
OBSOLETE_STATES = (
    ('CM', 'Commonwealth of the Northern Mariana Islands'),  # Is now 'MP'
    ('CZ', 'Panama Canal Zone'),                             # Reverted to Panama 1979
    ('PI', 'Philippine Islands'),                            # Philippine independence 1946
    ('TT', 'Trust Territory of the Pacific Islands'),        # Became the independent COFA states + Northern Mariana Islands 1979-1994
)


#: All US states and territories plus DC and military mail.
STATE_CHOICES = tuple(sorted(US_STATES + US_TERRITORIES + ARMED_FORCES_STATES, key=lambda obj: obj[1]))

#: All US Postal Service locations.
USPS_CHOICES = tuple(sorted(US_STATES + US_TERRITORIES + ARMED_FORCES_STATES + COFA_STATES, key=lambda obj: obj[1]))

#: Normalized versions of state names
STATES_NORMALIZED = {
    'ak': 'AK',
    'al': 'AL',
    'ala': 'AL',
    'alabama': 'AL',
    'alaska': 'AK',
    'american samao': 'AS',
    'american samoa': 'AS',
    'ar': 'AR',
    'ariz': 'AZ',
    'arizona': 'AZ',
    'ark': 'AR',
    'arkansas': 'AR',
    'as': 'AS',
    'az': 'AZ',
    'ca': 'CA',
    'calf': 'CA',
    'calif': 'CA',
    'california': 'CA',
    'co': 'CO',
    'colo': 'CO',
    'colorado': 'CO',
    'conn': 'CT',
    'connecticut': 'CT',
    'ct': 'CT',
    'dc': 'DC',
    'de': 'DE',
    'del': 'DE',
    'delaware': 'DE',
    'deleware': 'DE',
    'district of columbia': 'DC',
    'fl': 'FL',
    'fla': 'FL',
    'florida': 'FL',
    'ga': 'GA',
    'georgia': 'GA',
    'gu': 'GU',
    'guam': 'GU',
    'hawaii': 'HI',
    'hi': 'HI',
    'ia': 'IA',
    'id': 'ID',
    'idaho': 'ID',
    'il': 'IL',
    'ill': 'IL',
    'illinois': 'IL',
    'in': 'IN',
    'ind': 'IN',
    'indiana': 'IN',
    'iowa': 'IA',
    'kan': 'KS',
    'kans': 'KS',
    'kansas': 'KS',
    'kentucky': 'KY',
    'ks': 'KS',
    'ky': 'KY',
    'la': 'LA',
    'louisiana': 'LA',
    'ma': 'MA',
    'maine': 'ME',
    'marianas islands': 'MP',
    'marianas islands of the pacific': 'MP',
    'marinas islands of the pacific': 'MP',
    'maryland': 'MD',
    'mass': 'MA',
    'massachusetts': 'MA',
    'massachussetts': 'MA',
    'md': 'MD',
    'me': 'ME',
    'mi': 'MI',
    'mich': 'MI',
    'michigan': 'MI',
    'minn': 'MN',
    'minnesota': 'MN',
    'miss': 'MS',
    'mississippi': 'MS',
    'missouri': 'MO',
    'mn': 'MN',
    'mo': 'MO',
    'mont': 'MT',
    'montana': 'MT',
    'mp': 'MP',
    'ms': 'MS',
    'mt': 'MT',
    'n d': 'ND',
    'n dak': 'ND',
    'n h': 'NH',
    'n j': 'NJ',
    'n m': 'NM',
    'n mex': 'NM',
    'nc': 'NC',
    'nd': 'ND',
    'ne': 'NE',
    'neb': 'NE',
    'nebr': 'NE',
    'nebraska': 'NE',
    'nev': 'NV',
    'nevada': 'NV',
    'new hampshire': 'NH',
    'new jersey': 'NJ',
    'new mexico': 'NM',
    'new york': 'NY',
    'nh': 'NH',
    'nj': 'NJ',
    'nm': 'NM',
    'nmex': 'NM',
    'north carolina': 'NC',
    'north dakota': 'ND',
    'northern mariana islands': 'MP',
    'nv': 'NV',
    'ny': 'NY',
    'oh': 'OH',
    'ohio': 'OH',
    'ok': 'OK',
    'okla': 'OK',
    'oklahoma': 'OK',
    'or': 'OR',
    'ore': 'OR',
    'oreg': 'OR',
    'oregon': 'OR',
    'pa': 'PA',
    'penn': 'PA',
    'pennsylvania': 'PA',
    'pr': 'PR',
    'puerto rico': 'PR',
    'rhode island': 'RI',
    'ri': 'RI',
    's dak': 'SD',
    'sc': 'SC',
    'sd': 'SD',
    'sdak': 'SD',
    'south carolina': 'SC',
    'south dakota': 'SD',
    'tenn': 'TN',
    'tennessee': 'TN',
    'territory of hawaii': 'HI',
    'tex': 'TX',
    'texas': 'TX',
    'tn': 'TN',
    'tx': 'TX',
    'us virgin islands': 'VI',
    'usvi': 'VI',
    'ut': 'UT',
    'utah': 'UT',
    'va': 'VA',
    'vermont': 'VT',
    'vi': 'VI',
    'viginia': 'VA',
    'virgin islands': 'VI',
    'virgina': 'VA',
    'virginia': 'VA',
    'vt': 'VT',
    'w va': 'WV',
    'wa': 'WA',
    'wash': 'WA',
    'washington': 'WA',
    'west virginia': 'WV',
    'wi': 'WI',
    'wis': 'WI',
    'wisc': 'WI',
    'wisconsin': 'WI',
    'wv': 'WV',
    'wva': 'WV',
    'wy': 'WY',
    'wyo': 'WY',
    'wyoming': 'WY',
}
